@@ -1,13 +1,15 @@
-from datetime import datetime
 import logging
 import string
-import uiautomator2
+from datetime import datetime
 from enum import Enum, auto
-from os import listdir, getcwd
+from os import getcwd, listdir
 from random import randint, uniform
 from re import search
 from subprocess import PIPE, run
 from time import sleep
+
+import uiautomator2
+
 from GramAddict.core.utils import random_sleep
 
 logger = logging.getLogger(__name__)
@@ -38,6 +40,7 @@ def get_device_info(device):
 
 class Timeout(Enum):
     ZERO = auto()
+    TINY = auto()
     SHORT = auto()
     MEDIUM = auto()
     LONG = auto()
@@ -70,21 +73,15 @@ class Direction(Enum):
 
 
 class DeviceFacade:
-    deviceV2 = None  # uiautomator2
-
     def __init__(self, device_id):
         self.device_id = device_id
-        device_ip = None
-        # self.deviceV2.debug = True
         try:
-            if True:
-                self.deviceV2 = (
-                    uiautomator2.connect()
-                    if device_id is None
-                    else uiautomator2.connect(device_id)
+            if device_id is None or "." not in device_id:
+                self.deviceV2 = uiautomator2.connect(
+                    "" if device_id is None else device_id
                 )
             else:
-                self.deviveV2 = uiautomator2.connect_adb_wifi(f"{device_ip}:5555")
+                self.deviceV2 = uiautomator2.connect_adb_wifi(f"{device_id}")
         except ImportError:
             raise ImportError("Please install uiautomator2: pip3 install uiautomator2")
 
@@ -108,24 +105,57 @@ class DeviceFacade:
         random_sleep()
 
     def start_screenrecord(self, output="debug_0000.mp4", fps=20):
+        import imageio
+
+        def _run_MOD(self):
+            from collections import deque
+
+            pipelines = [self._pipe_limit, self._pipe_convert, self._pipe_resize]
+            _iter = self._iter_minicap()
+            for p in pipelines:
+                _iter = p(_iter)
+
+            with imageio.get_writer(self._filename, fps=self._fps) as wr:
+                frames = deque(maxlen=self._fps * 20)
+                for im in _iter:
+                    frames.append(im)
+                if self.crash:
+                    for frame in frames:
+                        wr.append_data(frame)
+            self._done_event.set()
+
+        def stop_MOD(self, crash=True):
+            """
+            stop record and finish write video
+            Returns:
+                bool: whether video is recorded.
+            """
+            if self._running:
+                self.crash = crash
+                self._stop_event.set()
+                ret = self._done_event.wait(10.0)
+
+                # reset
+                self._stop_event.clear()
+                self._done_event.clear()
+                self._running = False
+                return ret
+
+        from uiautomator2 import screenrecord as _sr
+
+        _sr.Screenrecord._run = _run_MOD
+        _sr.Screenrecord.stop = stop_MOD
         mp4_files = [f for f in listdir(getcwd()) if f.endswith(".mp4")]
         if mp4_files != []:
             last_mp4 = mp4_files[-1]
             debug_number = "{0:0=4d}".format(int(last_mp4[-8:-4]) + 1)
             output = f"debug_{debug_number}.mp4"
         self.deviceV2.screenrecord(output, fps)
-        logger.warning(
-            f"Start screen recording: it will be saved as '{output}' in '{getcwd()}'."
-        )
+        logger.warning("Screen recording has been started.")
 
-    def stop_screenrecord(self):
-        if self.deviceV2.screenrecord.stop():
-            mp4_files = [f for f in listdir(getcwd()) if f.endswith(".mp4")]
-            if mp4_files != []:
-                last_mp4 = mp4_files[-1]
-                logger.warning(
-                    f"Screen recorder has been stopped succesfully! File '{last_mp4}' available in '{getcwd()}'."
-                )
+    def stop_screenrecord(self, crash=True):
+        if self.deviceV2.screenrecord.stop(crash=crash):
+            logger.warning("Screen recorder has been stopped succesfully!")
 
     def screenshot(self, path):
         self.deviceV2.screenshot(path)
@@ -149,7 +179,7 @@ class DeviceFacade:
         )
         if data != "":
             flag = search("mDreamingLockscreen=(true|false)", data.stdout)
-            return True if flag is not None and flag.group(1) == "true" else False
+            return flag is not None and flag.group(1) == "true"
         else:
             logger.debug(
                 f"'adb -s {self.deviceV2.serial} shell dumpsys window' returns nothing!"
@@ -166,7 +196,7 @@ class DeviceFacade:
         )
         if data != "":
             flag = search("mInputShown=(true|false)", data.stdout)
-            return True if flag.group(1) == "true" else False
+            return flag.group(1) == "true"
         else:
             logger.debug(
                 f"'adb -s {serial} shell dumpsys input_method' returns nothing!"
@@ -187,9 +217,11 @@ class DeviceFacade:
     def unlock(self):
         self.swipe(Direction.UP, 0.8)
         sleep(2)
+        logger.debug(f"Screen locked: {self.is_screen_locked()}")
         if self.is_screen_locked():
             self.swipe(Direction.RIGHT, 0.8)
             sleep(2)
+            logger.debug(f"Screen locked: {self.is_screen_locked()}")
 
     def screen_off(self):
         self.deviceV2.screen_off()
@@ -231,11 +263,13 @@ class DeviceFacade:
 
     def swipe_points(self, sx, sy, ex, ey, random_x=True, random_y=True):
         if random_x:
-            sx = sx * uniform(0.60, 1.40)
-            ex = ex * uniform(0.85, 1.15)
+            sx = int(sx * uniform(0.85, 1.15))
+            ex = int(ex * uniform(0.85, 1.15))
         if random_y:
-            ey = ey * uniform(0.98, 1.02)
+            ey = int(ey * uniform(0.98, 1.02))
+        sy = int(sy)
         try:
+            logger.debug(f"Swipe from: ({sx},{sy}) to ({ex},{ey}).")
             self.deviceV2.swipe_points([[sx, sy], [ex, ey]], uniform(0.2, 0.5))
             DeviceFacade.sleep_mode(SleepTime.TINY)
         except uiautomator2.JSONRPCError as e:
@@ -409,7 +443,10 @@ class DeviceFacade:
         def click_retry(self, mode=None, sleep=None, coord=[], maxretry=2):
             """return True if successfully open the element, else False"""
             self.click(mode, sleep, coord)
+
             while maxretry > 0:
+                # we wait a little bit more before try again
+                random_sleep(2, 4, modulable=False)
                 if not self.exists():
                     return True
                 logger.debug("UI element didn't open! Try again..")
@@ -417,9 +454,8 @@ class DeviceFacade:
                 maxretry -= 1
             if not self.exists():
                 return True
-            else:
-                logger.warning("Failed to open the UI element!")
-                return False
+            logger.warning("Failed to open the UI element!")
+            return False
 
         def double_click(self, padding=0.3, obj_over=0):
             """Double click randomly in the selected view using padding
@@ -486,13 +522,16 @@ class DeviceFacade:
                 if self.viewV2 is None:
                     return False
                 exists = self.viewV2.exists(self.get_ui_timeout(ui_timeout))
-                if hasattr(self.viewV2, "count"):
-                    if not exists and self.viewV2.count >= 1:
-                        logger.debug(
-                            f"BUG: exists return False, but there is/are {self.viewV2.count} element(s)!"
-                        )
-                        # More info about that: https://github.com/openatx/uiautomator2/issues/689"
-                        return False
+                if (
+                    hasattr(self.viewV2, "count")
+                    and not exists
+                    and self.viewV2.count >= 1
+                ):
+                    logger.debug(
+                        f"BUG: exists return False, but there is/are {self.viewV2.count} element(s)!"
+                    )
+                    # More info about that: https://github.com/openatx/uiautomator2/issues/689"
+                    return False
                 return exists
             except uiautomator2.JSONRPCError as e:
                 raise DeviceFacade.JsonRpcError(e)
@@ -521,11 +560,19 @@ class DeviceFacade:
             except uiautomator2.JSONRPCError as e:
                 raise DeviceFacade.JsonRpcError(e)
 
+        def get_property(self, property):
+            try:
+                return self.viewV2.info[property]
+            except uiautomator2.JSONRPCError as e:
+                raise DeviceFacade.JsonRpcError(e)
+
         @staticmethod
         def get_ui_timeout(ui_timeout):
             ui_timeout = Timeout.ZERO if ui_timeout is None else ui_timeout
             if ui_timeout == Timeout.ZERO:
                 ui_timeout = 0
+            elif ui_timeout == Timeout.TINY:
+                ui_timeout = 1
             elif ui_timeout == Timeout.SHORT:
                 ui_timeout = 3
             elif ui_timeout == Timeout.MEDIUM:
@@ -545,13 +592,12 @@ class DeviceFacade:
                         if index is None
                         else self.viewV2[index].info["text"]
                     )
-                    if text is None:
-                        logger.debug(
-                            "Could not get text. Waiting 2 seconds and trying again..."
-                        )
-                        sleep(2)  # wait 2 seconds and retry
-                    else:
+                    if text is not None:
                         return text
+                    logger.debug(
+                        "Could not get text. Waiting 2 seconds and trying again..."
+                    )
+                    sleep(2)  # wait 2 seconds and retry
                 except uiautomator2.JSONRPCError as e:
                     if error:
                         raise DeviceFacade.JsonRpcError(e)
@@ -578,8 +624,7 @@ class DeviceFacade:
                 word_list = text.split()
                 n_words = len(word_list)
                 i = 0
-                n = 1
-                for word in word_list:
+                for n, word in enumerate(word_list, start=1):
                     n_single_letters = randint(1, 3)
                     for char in word:
                         if i < n_single_letters:
@@ -591,25 +636,17 @@ class DeviceFacade:
                                 self.deviceV2.send_keys(word[i:-1], clear=False)
                                 random_sleep(0.01, 0.1, modulable=False, logging=False)
                                 self.deviceV2.send_keys(word[-1], clear=False)
-                                random_sleep(0.01, 0.1, modulable=False, logging=False)
                             else:
                                 self.deviceV2.send_keys(word[i:], clear=False)
-                                random_sleep(0.01, 0.1, modulable=False, logging=False)
+                            random_sleep(0.01, 0.1, modulable=False, logging=False)
                             break
                     if n < n_words:
                         self.deviceV2.send_keys(" ", clear=False)
                         random_sleep(0.01, 0.1, modulable=False, logging=False)
 
                     i = 0
-                    n += 1
                 typed_text = self.viewV2.get_text()
-                if (
-                    typed_text is None
-                    or typed_text == "Add a comment…"
-                    or typed_text == "Message…"
-                    or typed_text == ""
-                    or typed_text.startswith("Comment as ")
-                ):
+                if typed_text != text:
                     logger.warning(
                         "Failed to write in text field, let's try in the old way.."
                     )
