@@ -1,21 +1,23 @@
-from time import sleep
-import emoji
-import yaml
-from GramAddict.core.utils import random_sleep
-from GramAddict.core.device_facade import Timeout
 import json
 import logging
 import os
 import re
-import unicodedata
 import sys
+import unicodedata
+from datetime import datetime
+from enum import Enum, auto
+from time import sleep
+from typing import Tuple
 
+import emoji
+import yaml
 from colorama import Fore
 from langdetect import detect
-from enum import Enum, auto
-from datetime import datetime
-from GramAddict.core.views import ProfileView, FollowStatus, OpenedPostView
-from GramAddict.core.resources import ClassName, ResourceID as resources
+
+from GramAddict.core.device_facade import Timeout
+from GramAddict.core.resources import ResourceID as resources
+from GramAddict.core.utils import random_sleep
+from GramAddict.core.views import FollowStatus, ProfileView
 
 logger = logging.getLogger(__name__)
 
@@ -23,6 +25,7 @@ FIELD_SKIP_BUSINESS = "skip_business"
 FIELD_SKIP_NON_BUSINESS = "skip_non_business"
 FIELD_SKIP_FOLLOWING = "skip_following"
 FIELD_SKIP_FOLLOWER = "skip_follower"
+FIELD_SKIP_IF_LINK_IN_BIO = "skip_if_link_in_bio"
 FIELD_MIN_FOLLOWERS = "min_followers"
 FIELD_MAX_FOLLOWERS = "max_followers"
 FIELD_MIN_FOLLOWINGS = "min_followings"
@@ -33,14 +36,17 @@ FIELD_FOLLOW_PRIVATE_OR_EMPTY = "follow_private_or_empty"
 FIELD_PM_TO_PRIVATE_OR_EMPTY = "pm_to_private_or_empty"
 FIELD_COMMENT_PHOTOS = "comment_photos"
 FIELD_COMMENT_VIDEOS = "comment_videos"
+FIELD_COMMENT_CAROUSELS = "comment_carousels"
 FIELD_INTERACT_ONLY_PRIVATE = "interact_only_private"
 FIELD_BLACKLIST_WORDS = "blacklist_words"
 FIELD_MANDATORY_WORDS = "mandatory_words"
 FIELD_SPECIFIC_ALPHABET = "specific_alphabet"
 FIELD_BIO_LANGUAGE = "biography_language"
+FIELD_BIO_BANNED_LANGUAGE = "biography_banned_language"
 FIELD_MIN_POSTS = "min_posts"
 FIELD_MIN_LIKERS = "min_likers"
 FIELD_MAX_LIKERS = "max_likers"
+FIELD_MUTUAL_FRIENDS = "mutual_friends"
 
 IGNORE_CHARSETS = ["MATHEMATICAL"]
 
@@ -75,29 +81,34 @@ class SkipReason(Enum):
     BIOGRAPHY_LANGUAGE_NOT_MATCH = auto()
     NOT_LOADED = auto()
     RESTRICTED = auto()
+    HAS_LINK_IN_BIO = auto()
+    LT_MUTUAL = auto()
 
 
 class Profile(object):
     def __init__(
         self,
+        mutual_friends,
         follow_button_text,
         is_restricted,
         is_private,
         has_business_category,
         posts_count,
         biography,
+        link_in_bio,
         fullname,
     ):
         self.datetime = str(datetime.now())
         self.followers = 0
         self.followings = 0
-
+        self.mutual_friends = mutual_friends
         self.follow_button_text = follow_button_text
         self.is_restricted = is_restricted
         self.is_private = is_private
         self.has_business_category = has_business_category
         self.posts_count = posts_count
         self.biography = biography
+        self.link_in_bio = link_in_bio
         self.fullname = fullname
 
     def set_followers_and_following(self, followers: int, followings: int):
@@ -125,7 +136,7 @@ class Filter:
                         logger.error(f"Error: {e}")
 
             elif os.path.exists(filter_path):
-                with open(filter_path) as json_file:
+                with open(filter_path, "r", encoding="utf-8") as json_file:
                     try:
                         self.conditions = json.load(json_file)
                         logger.warning(
@@ -139,31 +150,12 @@ class Filter:
                         sys.exit(0)
             else:
                 logger.warning(
-                    f"The filters file {filter_path} doesn't exists. Download it from https://github.com/GramAddict/bot/blob/08e1d7aff39ec47543fa78aadd7a2f034b9ae34d/config-examples/filters.yml and place it in your account folder!"
+                    f"The legacy filters file {filter_path} doesn't exists. Download and use the filters.yml instead from https://github.com/GramAddict/bot/blob/08e1d7aff39ec47543fa78aadd7a2f034b9ae34d/config-examples/filters.yml and place it in your account folder!"
                 )
         else:
             logger.warning("Filters are disabled!")
         self.storage = storage
 
-    def check_profile_from_list(self, device, item, username):
-        """check profile from list without open it"""
-        if self.conditions is None:
-            return True
-
-        field_skip_following = self.conditions.get(FIELD_SKIP_FOLLOWING, False)
-
-        if field_skip_following:
-            following = OpenedPostView(device)._isFollowing(item)
-
-            if following:
-                logger.info(
-                    f"You follow @{username}, skip.",
-                    extra={"color": f"{Fore.GREEN}"},
-                )
-                return False
-
-        return True
-
     def is_num_likers_in_range(self, likes_on_post):
         if self.conditions is not None and likes_on_post is not None:
             if likes_on_post == -1:
@@ -216,7 +208,12 @@ class Filter:
             )
             field_specific_alphabet = self.conditions.get(FIELD_SPECIFIC_ALPHABET)
             field_bio_language = self.conditions.get(FIELD_BIO_LANGUAGE)
+            field_bio_banned_language = self.conditions.get(FIELD_BIO_BANNED_LANGUAGE)
             field_min_posts = self.conditions.get(FIELD_MIN_POSTS)
+            field_mutual_friends = self.conditions.get(FIELD_MUTUAL_FRIENDS, -1)
+            field_skip_if_link_in_bio = self.conditions.get(
+                FIELD_SKIP_IF_LINK_IN_BIO, False
+            )
 
         profile_data = self.get_all_data(device)
         if self.conditions is None:
@@ -347,6 +344,22 @@ class Filter:
                 username, profile_data, SkipReason.UNDEFINED_FOLLOWERS_FOLLOWING
             )
 
+        if field_mutual_friends > -1:
+            logger.debug(
+                f"Checking if that user has at least {field_mutual_friends} mutual friends."
+            )
+            if profile_data.mutual_friends < field_mutual_friends:
+                return profile_data, self.return_check_profile(
+                    username, profile_data, SkipReason.LT_MUTUAL
+                )
+
+        if field_skip_if_link_in_bio:
+            logger.debug("Checking if account has link in bio...")
+            if profile_data.link_in_bio is not None:
+                return profile_data, self.return_check_profile(
+                    username, profile_data, SkipReason.HAS_LINK_IN_BIO
+                )
+
         if field_skip_business or field_skip_non_business:
             logger.debug("Checking if account is a business...")
             if field_skip_business and profile_data.has_business_category is True:
@@ -384,6 +397,7 @@ class Filter:
             or len(field_mandatory_words) > 0
             or field_specific_alphabet is not None
             or field_bio_language is not None
+            or field_bio_banned_language is not None
         ) and cleaned_biography != "":
             logger.debug("Pulling biography...")
             if len(field_blacklist_words) > 0:
@@ -435,14 +449,32 @@ class Filter:
                     return profile_data, self.return_check_profile(
                         username, profile_data, SkipReason.ALPHABET_NOT_MATCH
                     )
-            if field_bio_language is not None:
+            if field_bio_language is not None or field_bio_banned_language is not None:
+                skip_1 = skip_2 = False
                 logger.debug("Checking main language of account biography...")
                 language = self._find_language(cleaned_biography)
-                if language not in field_bio_language and language != "":
+                if (
+                    field_bio_banned_language
+                    and language in field_bio_banned_language
+                    and language != ""
+                ):
                     logger.info(
-                        f"@{username}'s biography language is not in {', '.join(field_bio_language)}. ({language}), skip.",
+                        f"@{username}'s biography language is in the banned list: {', '.join(field_bio_banned_language)}. ({language}), skip.",
                         extra={"color": f"{Fore.CYAN}"},
                     )
+                    skip_1 = True
+                if (
+                    not skip_1
+                    and field_bio_language
+                    and language not in field_bio_language
+                    and language != ""
+                ):
+                    logger.info(
+                        f"@{username}'s biography language is not in the list: {', '.join(field_bio_language)}. ({language}), skip.",
+                        extra={"color": f"{Fore.CYAN}"},
+                    )
+                    skip_2 = True
+                if skip_1 or skip_2:
                     return profile_data, self.return_check_profile(
                         username,
                         profile_data,
@@ -487,18 +519,17 @@ class Filter:
             field_pm_to_private_or_empty
         )
 
-    def can_comment(self, current_mode):
-        try:
+    def can_comment(self, current_mode) -> Tuple[bool, bool, bool, bool]:
+        if self.conditions is not None:
             return (
                 self.conditions.get(FIELD_COMMENT_PHOTOS, True),
                 self.conditions.get(FIELD_COMMENT_VIDEOS, True),
+                self.conditions.get(FIELD_COMMENT_CAROUSELS, True),
                 self.conditions.get("comment_" + current_mode.replace("-", "_"), False),
             )
-        except Exception as e:
-            logger.debug(
-                f"filters.yml (or legacy filter.json) is not loaded! Exception: {e}"
-            )
-            return False, False, False
+        else:
+            logger.debug("filters.yml (or legacy filter.json) is not loaded!")
+        return False, False, False, False
 
     def get_all_data(self, device):
         profile_picture = device.find(
@@ -529,24 +560,28 @@ class Filter:
         profileView = ProfileView(device)
         if not is_restricted:
             profile = Profile(
+                mutual_friends=self._get_mutual_friends(device, profileView),
                 follow_button_text=self._get_follow_button_text(device, profileView),
                 is_restricted=is_restricted,
                 is_private=self._is_private_account(device, profileView),
                 has_business_category=self._has_business_category(device, profileView),
                 posts_count=self._get_posts_count(device, profileView),
                 biography=self._get_profile_biography(device, profileView),
+                link_in_bio=self._get_link_in_bio(device, profileView),
                 fullname=self._get_fullname(device, profileView),
             )
             followers, following = self._get_followers_and_followings(device)
             profile.set_followers_and_following(followers, following)
         else:
             profile = Profile(
+                mutual_friends=None,
                 follow_button_text=None,
                 is_restricted=is_restricted,
                 is_private=None,
                 has_business_category=None,
                 posts_count=None,
                 biography=None,
+                link_in_bio=None,
                 fullname=None,
             )
             profile.set_followers_and_following(None, None)
@@ -577,7 +612,6 @@ class Filter:
     def _has_business_category(device, profileView=None):
         business_category_view = device.find(
             resourceId=ResourceID.PROFILE_HEADER_BUSINESS_CATEGORY,
-            className=ClassName.TEXT_VIEW,
         )
         return business_category_view.exists()
 
@@ -661,3 +695,15 @@ class Filter:
         profileView = ProfileView(device) if profileView is None else profileView
         _, text = profileView.getFollowButton()
         return text
+
+    @staticmethod
+    def _get_mutual_friends(device, profileView=None):
+        profileView = ProfileView(device) if profileView is None else profileView
+        mutual_friends = profileView.getMutualFriends()
+        return mutual_friends
+
+    @staticmethod
+    def _get_link_in_bio(device, profileView=None):
+        profileView = ProfileView(device) if profileView is None else profileView
+        link = profileView.getLinkInBio()
+        return link
