@@ -1,9 +1,11 @@
 import json
 import logging
 import os
+import sys
 from datetime import datetime, timedelta
 from enum import Enum, unique
-import sys
+
+from atomicwrites import atomic_write
 
 logger = logging.getLogger(__name__)
 
@@ -52,7 +54,7 @@ class Storage:
             f"{ACCOUNTS}/{my_username}/{FILENAME_INTERACTED_USERS}"
         )
         if os.path.exists(self.interacted_users_path):
-            with open(self.interacted_users_path) as json_file:
+            with open(self.interacted_users_path, encoding="utf-8") as json_file:
                 try:
                     self.interacted_users = json.load(json_file)
                 except Exception as e:
@@ -66,7 +68,7 @@ class Storage:
         )
 
         if os.path.exists(self.history_filter_users_path):
-            with open(self.history_filter_users_path) as json_file:
+            with open(self.history_filter_users_path, encoding="utf-8") as json_file:
                 try:
                     self.history_filter_users = json.load(json_file)
                 except Exception as e:
@@ -81,28 +83,36 @@ class Storage:
 
         whitelist_path = f"{ACCOUNTS}/{my_username}/{FILENAME_WHITELIST}"
         if os.path.exists(whitelist_path):
-            with open(whitelist_path) as file:
+            with open(whitelist_path, encoding="utf-8") as file:
                 self.whitelist = [line.rstrip() for line in file]
 
         blacklist_path = f"{ACCOUNTS}/{my_username}/{FILENAME_BLACKLIST}"
         if os.path.exists(blacklist_path):
-            with open(blacklist_path) as file:
+            with open(blacklist_path, encoding="utf-8") as file:
                 self.blacklist = [line.rstrip() for line in file]
 
         self.report_path = f"{ACCOUNTS}/{my_username}/{REPORTS}/"
 
-    def check_user_was_interacted(self, username):
-        return not self.interacted_users.get(username) is None
+    def can_be_reinteract(self, last_interaction, hours_that_have_to_pass):
+        if hours_that_have_to_pass > timedelta(hours=0):
+            delta = datetime.now() - last_interaction
+            if delta < hours_that_have_to_pass:
+                return False
+            else:
+                return True
+        else:
+            return False
 
-    def check_user_was_interacted_recently(self, username):
+    def check_user_was_interacted(self, username):
+        """returns when an username has been interacted, False if not already interacted"""
         user = self.interacted_users.get(username)
         if user is None:
-            return False
+            return False, None
 
         last_interaction = datetime.strptime(
             user[USER_LAST_INTERACTION], "%Y-%m-%d %H:%M:%S.%f"
         )
-        return datetime.now() - last_interaction <= timedelta(days=3)
+        return True, last_interaction
 
     def get_following_status(self, username):
         user = self.interacted_users.get(username)
@@ -121,7 +131,9 @@ class Storage:
         user["skip_reason"] = None if skip_reason is None else skip_reason.name
         self.history_filter_users[username] = user
         if self.history_filter_users_path is not None:
-            with open(self.history_filter_users_path, "w") as outfile:
+            with atomic_write(
+                self.history_filter_users_path, overwrite=True, encoding="utf-8"
+            ) as outfile:
                 json.dump(self.history_filter_users, outfile, indent=4, sort_keys=False)
 
     def add_interacted_user(
@@ -154,8 +166,10 @@ class Storage:
         user["session_id"] = session_id
 
         # Save only the last job_name and target
-        user["job_name"] = job_name
-        user["target"] = target
+        if not user.get("job_name"):
+            user["job_name"] = job_name
+        if not user.get("target"):
+            user["target"] = target
 
         # Increase the value of liked, watched or commented if we have already a value
         user["liked"] = liked if "liked" not in user else (user["liked"] + liked)
@@ -211,7 +225,9 @@ class Storage:
 
     def _update_file(self):
         if self.interacted_users_path is not None:
-            with open(self.interacted_users_path, "w") as outfile:
+            with atomic_write(
+                self.interacted_users_path, overwrite=True, encoding="utf-8"
+            ) as outfile:
                 json.dump(self.interacted_users, outfile, indent=4, sort_keys=False)
 
 
