@@ -1,29 +1,29 @@
-from GramAddict import __file__
-from GramAddict.core.storage import ACCOUNTS
-from math import nan
-import random
-import sys
-
-from GramAddict.core.report import print_full_report
 import logging
 import os
-import subprocess
+import random
 import re
 import shutil
-import urllib3
-import emoji
-from os import getcwd, walk, rename
-from pathlib import Path
+import subprocess
+import sys
 from datetime import datetime
+from math import nan
+from os import getcwd, rename, walk
+from pathlib import Path
 from random import randint, shuffle, uniform
 from subprocess import PIPE
 from time import sleep
 from urllib.parse import urlparse
+
+import emoji
+import urllib3
+from colorama import Fore, Style
 from requests import get
 
-from colorama import Fore, Style
+from GramAddict import __file__
 from GramAddict.core.log import get_log_file_config
+from GramAddict.core.report import print_full_report
 from GramAddict.core.resources import ResourceID as resources
+from GramAddict.core.storage import ACCOUNTS
 from GramAddict.version import __version__
 
 http = urllib3.PoolManager()
@@ -48,7 +48,7 @@ def update_available():
     else:
         version_request = "https://raw.githubusercontent.com/GramAddict/bot/develop/GramAddict/version.py"
     try:
-        r = get(version_request, verify=False)
+        r = get(version_request, verify=True)
         online_version_raw = r.text.split('"')[1]
 
     except Exception as e:
@@ -69,10 +69,9 @@ def update_available():
     for n in range(len(online_version)):
         if int(online_version[n]) > int(local_version[n]):
             return True, online_version_raw
-        else:
-            if int(online_version[n]) == int(local_version[n]):
-                continue
-            break
+        if int(online_version[n]) == int(local_version[n]):
+            continue
+        break
     return False, online_version_raw
 
 
@@ -89,9 +88,8 @@ def check_if_updated(crash=False):
         logger.warning("If you installed with pip: pip3 install GramAddict -U")
         logger.warning("If you installed with git: git pull")
         sleep(5)
-    else:
-        if not crash:
-            logger.info("Bot is updated.", extra={"color": f"{Style.BRIGHT}"})
+    elif not crash:
+        logger.info("Bot is updated.", extra={"color": f"{Style.BRIGHT}"})
     if not crash:
         logger.info(
             f"GramAddict v.{__version__}",
@@ -125,6 +123,7 @@ def move_usernames_to_accounts():
         "logs",
         "res",
         "test",
+        "dump",
     ]
     for n in ignored_dir:
         try:
@@ -195,15 +194,12 @@ def get_instagram_version():
     )
     output = stream.read()
     version_match = re.findall("versionName=(\\S+)", output)
-    if len(version_match) == 1:
-        version = version_match[0]
-    else:
-        version = "not found"
+    version = version_match[0] if len(version_match) == 1 else "not found"
     stream.close()
     return version
 
 
-def open_instagram_with_url(url):
+def open_instagram_with_url(url) -> bool:
     logger.info("Open Instagram app with url: {}".format(url))
     cmd = (
         "adb"
@@ -219,9 +215,23 @@ def open_instagram_with_url(url):
     return True
 
 
+def open_app(device, app_id):
+    device.deviceV2.app_start(app_id, use_monkey=True)
+    for _ in range(3):
+        if device.deviceV2.app_wait(app_id, 5, True):
+            return True
+        kill_app(device, app_id)
+        device.deviceV2.app_start(app_id, use_monkey=True)
+    return device.deviceV2.app_wait(app_id, 5, True)
+
+
+def kill_app(device, app_id):
+    device.deviceV2.app_stop(app_id)
+
+
 def open_instagram(device, screen_record, close_apps):
-    FastInputIME = "com.github.uiautomator/.FastInputIME"
     nl = "\n"
+    FastInputIME = "com.github.uiautomator/.FastInputIME"
     logger.info("Open Instagram app.")
     cmd = (
         "adb"
@@ -231,7 +241,7 @@ def open_instagram(device, screen_record, close_apps):
     cmd_res = subprocess.run(cmd, stdout=PIPE, stderr=PIPE, shell=True, encoding="utf8")
     err = cmd_res.stderr.strip()
     if "Error" in err:
-        logger.error(err.replace("\n", ". "))
+        logger.error(err.replace(nl, ". "))
         return False
     elif "more than one device/emulator" in err:
         logger.error(
@@ -242,23 +252,23 @@ def open_instagram(device, screen_record, close_apps):
         logger.debug("Instagram called succesfully.")
     else:
         logger.debug(f"{err.replace('Warning: ', '')}.")
-    success = False
-    for _ in range(3):
-        if device.deviceV2.info["currentPackageName"] == app_id:
-            success = True
-            break
-        logger.debug("Wait for Instagram to open.")
-        sleep(3)
-    if success:
-        logger.info(
-            "Ready for botting!🤫", extra={"color": f"{Style.BRIGHT}{Fore.GREEN}"}
-        )
-    else:
-        logger.error("Unabled to open Instagram. Try again..")
-        return False
+
+    max_tries = 3
+    n = 0
+    while device.deviceV2.info["currentPackageName"] != app_id:
+        if n > max_tries:
+            logger.critical("Unabled to open Instagram. Bot will stop.")
+            return False
+        n += 1
+        logger.info(f"Waiting for Instagram to open... 😴 ({n}/{max_tries})")
+        check_if_crash_popup_is_there(device)
+        random_sleep(3, 3, modulable=False)
+
+    logger.info("Ready for botting!🤫", extra={"color": f"{Style.BRIGHT}{Fore.GREEN}"})
+
     random_sleep()
     if close_apps:
-        logger.info("Close all the other apps, for avoid interferece..")
+        logger.info("Close all the other apps, to avoid interferences...")
         device.deviceV2.app_stop_all(excludes=[app_id])
         random_sleep()
     logger.debug("Setting FastInputIME as default keyboard.")
@@ -294,7 +304,7 @@ def open_instagram(device, screen_record, close_apps):
             device.start_screenrecord()
         except Exception as e:
             logger.error(
-                f"For use the screen-record feature you have to install the requirements package! Run in the console: 'pip3 install -U 'uiautomator2[image]' -i https://pypi.doubanio.com/simple' Exception: {e}"
+                f"You can't use this feature without installing dependencies. Type that in console: 'pip3 install -U \"uiautomator2[image]\" -i https://pypi.doubanio.com/simple'. Exception: {e}"
             )
     return True
 
@@ -302,23 +312,44 @@ def open_instagram(device, screen_record, close_apps):
 def close_instagram(device, screen_record):
     logger.info("Close Instagram app.")
     device.deviceV2.app_stop(app_id)
+    random_sleep(5, 5, modulable=False)
     if screen_record:
         try:
-            device.stop_screenrecord()
+            device.stop_screenrecord(crash=False)
         except Exception as e:
             logger.error(
-                f"For use the screen-record feature you have to install the requirements package! Run in the console: 'pip3 install -U 'uiautomator2[image]' -i https://pypi.doubanio.com/simple' Exception: {e}"
+                f"You can't use this feature without installing dependencies. Type that in console: 'pip3 install -U \"uiautomator2[image]\" -i https://pypi.doubanio.com/simple'. Exception: {e}"
+            )
+
+
+def check_if_crash_popup_is_there(device):
+    obj = device.find(resourceId=ResourceID.AERR_RESTART)
+    if obj.exists():
+        obj.click()
+
+
+def pre_post_script(path: str, pre: bool = True):
+    if path is not None:
+        if os.path.isfile(path):
+            logger.info(f"Running '{path}' as {'pre' if pre else 'post'} script.")
+            try:
+                p1 = subprocess.Popen(path)
+                p1.wait()
+            except Exception as ex:
+                logger.error(f"This exception has occurred: {ex}")
+        else:
+            logger.error(
+                f"File '{path}' not found. Check your spelling. (The start point for relative paths is this: '{os.getcwd()}')."
             )
 
 
 def print_telegram_reports(
     configs, telegram_reports_at_end, followers_now, following_now, time_left=None
 ):
-    if followers_now is not None:
-        if telegram_reports_at_end:
-            configs.actions["telegram-reports"].run(
-                configs, "telegram-reports", followers_now, following_now, time_left
-            )
+    if followers_now is not None and telegram_reports_at_end:
+        configs.actions["telegram-reports"].run(
+            configs, "telegram-reports", followers_now, following_now, time_left
+        )
 
 
 def kill_atx_agent(device):
@@ -336,8 +367,7 @@ def random_sleep(inf=0.5, sup=3.0, modulable=True, logging=True):
     MIN_INF = 0.3
     multiplier = float(args.speed_multiplier)
     delay = uniform(inf, sup) / (multiplier if modulable else 1.0)
-    if delay < MIN_INF:
-        delay = MIN_INF
+    delay = max(delay, MIN_INF)
     if logging:
         logger.debug(f"{str(delay)[0:4]}s sleep")
     sleep(delay)
@@ -345,39 +375,40 @@ def random_sleep(inf=0.5, sup=3.0, modulable=True, logging=True):
 
 def save_crash(device):
     directory_name = __version__ + "_" + datetime.now().strftime("%Y-%m-%d-%H-%M-%S")
+    crash_path = os.path.join("crashes", directory_name)
     try:
-        os.makedirs("crashes/" + directory_name + "/", exist_ok=False)
+        os.makedirs(crash_path, exist_ok=False)
     except OSError:
-        logger.error("Directory " + directory_name + " already exists.")
+        logger.error(f"Directory {directory_name} already exists.")
         return
     screenshot_format = ".png"
     try:
-        device.screenshot(
-            "crashes/" + directory_name + "/screenshot" + screenshot_format
-        )
+        device.screenshot(os.path.join(crash_path, "screenshot" + screenshot_format))
     except RuntimeError:
-        logger.error("Cannot save screenshot.")
+        logger.error(f"Cannot save 'screenshot.{screenshot_format}'.")
 
-    view_hierarchy_format = ".xml"
+    hierarchy_format = ".xml"
     try:
-        device.dump_hierarchy(
-            "crashes/" + directory_name + "/view_hierarchy" + view_hierarchy_format
-        )
+        device.dump_hierarchy(os.path.join(crash_path, "hierarchy" + hierarchy_format))
     except RuntimeError:
-        logger.error("Cannot save view hierarchy.")
-
+        logger.error(f"Cannot save 'hierarchy.{hierarchy_format}'.")
+    if args.screen_record:
+        device.stop_screenrecord()
+        files = [f for f in os.listdir("./") if f.endswith(".mp4")]
+        try:
+            os.replace(files[-1], os.path.join(crash_path, "video.mp4"))
+        except (FileNotFoundError, IndexError):
+            logger.error("File *.mp4 not found!")
     g_log_file_name, g_logs_dir, _, _ = get_log_file_config()
-    src_file = f"{g_logs_dir}/{g_log_file_name}"
-    target_file = f"crashes/{directory_name}/logs.txt"
+    src_file = os.path.join(g_logs_dir, g_log_file_name)
+    target_file = os.path.join(crash_path, "logs.txt")
     shutil.copy(src_file, target_file)
 
-    shutil.make_archive(
-        "crashes/" + directory_name, "zip", "crashes/" + directory_name + "/"
-    )
-    shutil.rmtree("crashes/" + directory_name + "/")
+    shutil.make_archive(crash_path, "zip", crash_path)
+    shutil.rmtree(crash_path)
 
     logger.info(
-        'Crash saved as "crashes/' + directory_name + '.zip".',
+        f"Crash saved as {crash_path}.zip",
         extra={"color": Fore.GREEN},
     )
     logger.info(
@@ -386,6 +417,8 @@ def save_crash(device):
     )
     logger.info("https://discord.gg/66zWWCDM7x\n", extra={"color": Fore.GREEN})
     check_if_updated(crash=True)
+    if args.screen_record:
+        device.start_screenrecord()
 
 
 def stop_bot(device, sessions, session_state, screen_record, was_sleeping=False):
@@ -404,21 +437,19 @@ def stop_bot(device, sessions, session_state, screen_record, was_sleeping=False)
 
 
 def can_repeat(current_session, max_sessions):
-    if max_sessions != -1:
-        logger.info(
-            f"You completed {current_session} session(s). {max_sessions-current_session} session(s) left.",
-            extra={"color": f"{Style.BRIGHT}{Fore.YELLOW}"},
-        )
-        if current_session >= max_sessions:
-            logger.info(
-                "You reached the total-sessions limit! Finish.",
-                extra={"color": f"{Style.BRIGHT}{Fore.YELLOW}"},
-            )
-            return False
-        else:
-            return True
-    else:
+    if max_sessions == -1:
         return True
+    logger.info(
+        f"You completed {current_session} session(s). {max_sessions-current_session} session(s) left.",
+        extra={"color": f"{Style.BRIGHT}{Fore.YELLOW}"},
+    )
+    if current_session < max_sessions:
+        return True
+    logger.info(
+        "You reached the total-sessions limit! Finish.",
+        extra={"color": f"{Style.BRIGHT}{Fore.YELLOW}"},
+    )
+    return False
 
 
 def get_value(count, name, default, its_time=False):
@@ -504,11 +535,20 @@ def sample_sources(sources, n_sources):
     return truncaded
 
 
+def random_choice(number: int) -> bool:
+    """
+    Generate a random int and compare with the argument passed
+    :param int number: number passed
+    :return: is argument greater or equal then a random generated number
+    :rtype: bool
+    """
+    return number >= randint(1, 100)
+
+
 def init_on_things(source, args, sessions, session_state):
     from functools import partial
-    from GramAddict.core.interaction import (
-        _on_interaction,
-    )
+
+    from GramAddict.core.interaction import _on_interaction
 
     on_interaction = partial(
         _on_interaction,
